package statistics

import (
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/model"
	"strings"
)

// GetUserTypes 获取人员类型
func (m *StatisticsDAO) GetUserTypes(id string) ([]model.SysUserType, error) {
	if id == "" {
		return nil, nil
	}

	var tps []model.SysUserType
	if err := m.db.Where("user_id=?", id).Find(&tps); err != nil {
		return nil, err
	}

	return tps, nil
}

// GetDashboardSetting 获取人员 dashboard 设置
func (m *StatisticsDAO) GetDashboardSetting(userID string, userTypes []string) ([]string, error) {
	filters := make([]string, 0)
	if userID != "" {
		filters = append(filters, "(for_type='"+models.DASH_SETTING_FOR_USER+"' AND owner='"+userID+"')")
	}

	if userTypes != nil {
		filters = append(filters, "(for_type='"+models.DASH_SETTING_FOR_USERTYPE+"' AND owner in ('"+strings.Join(userTypes, "','")+"'))")
	}

	if len(filters) == 0 {
		return nil, nil
	}

	sql := `select DISTINCT comp_id from ta_dashboard_setting where (` + strings.Join(filters, " OR ") + ") AND status = ? order by sort_no asc"
	res, err := m.db.Query(sql, models.STATUS_NORMAL)
	if err != nil {
		return nil, err
	}

	if res == nil || len(res) == 0 {
		return nil, nil
	}

	settings := make([]string, 0)
	for _, r := range res {
		settings = append(settings, string(r["comp_id"]))
	}

	return settings, nil
}

// GetAllFrontComponents 获取所有前端支持组件
func (m *StatisticsDAO) GetAllFrontComponents() ([]model.TdStatisticsComps, error) {
	var comps []model.TdStatisticsComps
	if err := m.db.Find(&comps); err != nil {
		return nil, err
	}

	return comps, nil
}
