package statistics

import (
	"spaceofcheng/services/utils"
	"strconv"
	"strings"
	"time"

	"github.com/go-xorm/xorm"
)

// StatisticsDAO 当前数据库操作对象
type StatisticsDAO struct {
	ctx *utils.Context
	db  *xorm.Session
}

// NewStatisticsDAO New Inst
func NewStatisticsDAO(ctx *utils.Context) *StatisticsDAO {
	return &StatisticsDAO{
		ctx: ctx,
		db:  ctx.DB,
	}
}

// GetCustomerNumber 获取会员数量
func (m *StatisticsDAO) GetCustomerNumber(orgID string, caseIDs []string, start, end *time.Time) (int64, error) {
	return m.execMySQLFunc("fun_total_customer", orgID, caseIDs, start, end)
}

// GetCourseOrderNumber 获取课程预约 - 统计课程
func (m *StatisticsDAO) GetCourseOrderNumber(orgID string, caseIDs []string, start, end *time.Time) (int64, error) {
	return m.execMySQLFunc("fun_course_order", orgID, caseIDs, start, end)
}

// GetCourseOrderCustomer 获取课程预约 - 统计人员
func (m *StatisticsDAO) GetCourseOrderCustomer(orgID string, caseIDs []string, start, end *time.Time) (int64, error) {
	return m.execMySQLFunc("fun_course_order_customer", orgID, caseIDs, start, end)
}

// GetOrderGoodsNumber 获取商品下单数量
func (m *StatisticsDAO) GetOrderGoodsNumber(orgID string, caseIDs []string, start, end *time.Time) (int64, error) {
	return m.execMySQLFunc("fun_order_goods_num", orgID, caseIDs, start, end)
}

// GetCourseNumber 获取课程数量
func (m *StatisticsDAO) GetCourseNumber(orgID string, caseIDs []string, start, end *time.Time) (int64, error) {
	return m.execMySQLFunc("fun_total_course", orgID, caseIDs, start, end)
}

// GetArrivedCustomerNumber 获取到场人数
func (m *StatisticsDAO) GetArrivedCustomerNumber(orgID string, caseIDs []string, start, end *time.Time) (int64, error) {
	return m.execMySQLFunc("fun_arrived_customer", orgID, caseIDs, start, end)
}

// GetCourseCardCustomer 获取预约人数 - 含健身卡
func (m *StatisticsDAO) GetCourseCardCustomer(orgID string, caseIDs []string, start, end *time.Time) (int64, error) {
	return m.execMySQLFunc("fun_course_card_customer", orgID, caseIDs, start, end)
}

// GetCouponCardNumber 获取卡券总数
func (m *StatisticsDAO) GetCouponCardNumber(orgID string, caseIDs []string, start, end *time.Time) (int64, error) {
	return m.execMySQLFunc("fun_total_coupon_card", orgID, caseIDs, start, end)
}

// GetUsedCouponCardNumber 获取已使用卡券
func (m *StatisticsDAO) GetUsedCouponCardNumber(orgID string, caseIDs []string, start, end *time.Time) (int64, error) {
	return m.execMySQLFunc("fun_used_coupon_card", orgID, caseIDs, start, end)
}

// GetCustomerCouponCardNumber 获取已发放卡券总数
func (m *StatisticsDAO) GetCustomerCouponCardNumber(orgID string, caseIDs []string, start, end *time.Time) (int64, error) {
	return m.execMySQLFunc("fun_total_customer_coupon_card", orgID, caseIDs, start, end)
}

// execMySQLFunc 执行 mysql 函数
func (m *StatisticsDAO) execMySQLFunc(funName, orgID string, caseIDs []string, start, end *time.Time) (int64, error) {
	csids := ""
	if caseIDs == nil && len(caseIDs) == 0 {
		csids = strings.Join(caseIDs, ",")
	}

	startDate := ""
	endDate := ""

	if start != nil {
		startDate = start.Format("2006-01-02 15:04:05")
	}
	if end != nil {
		endDate = end.Format("2006-01-02 15:04:05")
	}

	sql := "select " + funName + "(?,?,?,?) as total"
	res, err := m.db.Query(sql, orgID, csids, startDate, endDate)
	if err != nil {
		return 0, err
	}

	if res == nil || len(res) == 0 {
		return 0, nil
	}

	total, _ := strconv.ParseInt(string(res[0]["total"]), 10, 64)
	return total, nil
}
